#!/usr/bin/python3

import json
import os
import psycopg2
import sys
from psycopg2.extras import RealDictCursor
from importlib.machinery import SourceFileLoader

def translateColToPg(column_name): # model/FeatureClass.js to postgres; currently no translation required
    return column_name

def translateValToPg(column_name, value):
    if (not value is None) and (str(value).strip() != ''):
        return "'" + str(value).strip() + "'"
    else:
        return "NULL"

def updateFeatureclasses(featureclassInserts, featureclassUpdates):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        sql = "BEGIN;"
        query.execute(sql)
        featureclasses = {}
        for featureclasskey in featureclassInserts.keys():
            sql = ""
            inserts = featureclassInserts[featureclasskey]
            for column_name in inserts.keys():
                if sql == "":
                    sql += "INSERT INTO fews.featureclass ("
                else:
                    sql += ", "
                sql += translateColToPg(column_name)
            sql += ") VALUES ("
            first = True
            for column_name in inserts.keys():
                if first:
                    first = False
                else:
                    sql += ", "
                sql += translateValToPg(column_name, inserts[column_name])
            sql += ") RETURNING featureclasskey;"
            query.execute(sql)
            featureclasskey = query.fetchone()['featureclasskey']
            featureclass = {}
            featureclass['featureclasskey'] = featureclasskey
            featureclasses[featureclasskey] = featureclass
        for featureclasskey in featureclassUpdates.keys():
            sql = ""
            updates = featureclassUpdates[featureclasskey]
            for column_name in updates.keys():
                if sql == "":
                    sql += "UPDATE fews.featureclass SET "
                else:
                    sql += ", "
                sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, updates[column_name])
            sql += " WHERE featureclasskey='" + str(featureclasskey) + "';"
            query.execute(sql)
        messages = list(featureclasses.values())
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys feature class!")
    return success, messages

contentlength = os.environ.get('CONTENT_LENGTH', 0)
postdata = sys.stdin.read(int(contentlength))
postdata = json.loads(postdata)
ticket = postdata['ticket']
curpath = os.path.dirname(os.path.realpath( __file__ ))
cas = SourceFileLoader("cas", curpath + "/../management/cas.py").load_module()
if cas.authorized(ticket,{"basedata" : "rw"}):
    featureclassUpdates = {}
    featureclassInserts = {}
    for record in postdata:
        if record == 'ticket':
            continue
        record = postdata[record]
        if 'featureclasskey' in record: # insert or update
            featureclasskey = record['featureclasskey']
            if featureclasskey < 0: # insert
                inserts = {}
                for key in record.keys():
                    if key != 'featureclasskey':
                        inserts[key] = record[key]
                featureclassInserts[featureclasskey] = inserts
            else: # update
                updates = {}
                for key in record.keys():
                    if key != 'featureclasskey':
                        updates[key] = record[key]
                featureclassUpdates[featureclasskey] = updates

    success, messages = updateFeatureclasses(featureclassInserts, featureclassUpdates)
else:
    success = False
    messages = ['Not authorized.']

print ("Content-type: application/json")
print ()
if success:
    print("{\"success\": true, \"messages\": " + json.dumps(messages) + "}")
else:
    print("{\"success\": false, \"messages\": " + json.dumps(messages) + "}")

