from datetime import datetime
import psycopg2
from psycopg2.extras import RealDictCursor

def title():
    return "Discharge"

def abstract():
    return "Gets a timesseries of water levels and determines the discharge, if the rating curves are available. Discharge is the volume of water moving down a stream or river per unit of time expressed in cubic meters per second."

def inputs():
    return [['startdate','Start date for data collection','string'],
            ['enddate','End date for data collection','string'],
            ['parameterid','Parameter ID','string'],
            ['locations','Locations selected for discharge calculation','string']]

def getCoefs(ratingcurve, date, waterlevel):
    i = len(ratingcurve) - 1
    while i >= 0 and date < ratingcurve[i]['start']:
        i = i - 1
    while i >= 0 and waterlevel < ratingcurve[i]['low']:
        i = i - 1
    if i >= 0:
        return ratingcurve[i]['a_coef'], ratingcurve[i]['b_coef'], ratingcurve[i]['c_coef']
    else:
        return None, None, None

def process(identifier, parameters, records):
    locationid = parameters.get('locationid')
    file = open("../.credentials/.postgres")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    sql = "SELECT start, low, a_coef, b_coef, c_coef \
            FROM ratingcurve.period JOIN ratingcurve.segment ON period.periodkey = segment.periodkey \
            JOIN fews.locations ON period.locationkey = locations.locationkey \
            WHERE locations.id = '" + locationid + "' \
            ORDER BY start, low"
    query.execute(sql)
    ratingcurve = query.fetchall()
    for record in records:
        date = record['date']
        waterlevel = record['value']
        Q, h0, alpha = getCoefs(ratingcurve, date, waterlevel)
        if not Q is None and h0 <= waterlevel:
            discharge = Q * (waterlevel - h0) ** alpha
        else:
            discharge = None
        record['value'] = discharge
        record['date'] = datetime.strftime(date, "%Y-%m-%d %H:%M:%S") # change datetime.datetime to str for json.dumps()
        
    return records
