import os, tempfile, stat, glob
from datetime import datetime
import psycopg2
from psycopg2.extras import RealDictCursor
from importlib.machinery import SourceFileLoader
import json, cgi

curpath = os.path.dirname(os.path.realpath( __file__ ))
jwt = SourceFileLoader("jwt", curpath + "/jwt.py").load_module()
session_timeout = 7200 # 2 hours
fileprefix = "majisys_session_"
file = open(curpath + "/../.credentials/.jwt")
jwt_secret = file.readline()

def makefilenamesafe(filename):
    return "".join(c for c in filename if c.isalnum()).rstrip() # filename-safe characters

def touch(filename):
    fhandle = open(filename, 'a')
    try:
        os.utime(filename)
    finally:
        fhandle.close()

def login(ticket):
    deleteexpiredsessionfiles()
    hash = ticket.split('.')[2]
    hash = makefilenamesafe(hash)
    touch(tempfile.gettempdir() + "/" + fileprefix + hash)

def renew(ticket):
    hash = ticket.split('.')[2]
    hash = makefilenamesafe(hash)
    sessionfile = tempfile.gettempdir() + "/" + fileprefix + hash
    if os.path.exists(sessionfile):
        touch(sessionfile)

def logout(ticket):
    hash = ticket.split('.')[2]
    hash = makefilenamesafe(hash)
    sessionfile = tempfile.gettempdir() + "/" + fileprefix + hash
    if os.path.exists(sessionfile):
        os.unlink(sessionfile)

def getdate(ticket):
    hash = ticket.split('.')[2]
    hash = makefilenamesafe(hash)
    sessionfile = tempfile.gettempdir() + "/" + fileprefix + hash
    if os.path.exists(sessionfile):
        return datetime.fromtimestamp(os.path.getmtime(sessionfile))
    else:
        return datetime.fromtimestamp(0) # epoch

def authorized(ticket, required_privileges):
    authorized = False
    try:
        header, payload, valid = jwt.jwt_decode(ticket, jwt_secret, 'HS256')
        valid = valid and not expired(ticket)
        payload = json.loads(payload)
        roleid = payload['roleid']
        ip = payload['ip']
        valid = valid and (ip == str(cgi.escape(os.environ["REMOTE_ADDR"])))
        if roleid == -1 and "setpasswd" in required_privileges.keys(): # set new passwd
            authorized = valid
        else:
            sql_code = "SELECT DISTINCT route, readwrite FROM app.role_navtree JOIN app.navtree USING (itemid) WHERE roleid=" + str(roleid)
            if len(required_privileges) > 0:
                sql_code += " AND ("
                first = True
                for module in required_privileges.keys():
                    if first:
                        first = False
                    else:
                        sql_code += " OR "
                    rw = " AND readwrite" if required_privileges[module] == "rw" else ""
                    sql_code += "(route='" + module + "'" + rw + ")"
                sql_code += ")"

            file = open(curpath + "/../.credentials/.postgres")
            connection_string = file.readline()
            pg = psycopg2.connect(connection_string)
            query = pg.cursor(cursor_factory=RealDictCursor)
            query.execute(sql_code)
            authorized = valid and (query.rowcount > 0)
            if authorized:
                renew(ticket)
    finally:
        return authorized

def valid(ticket):
    valid = False
    try:
        valid = jwt.jwt_decode(ticket, jwt_secret, 'HS256')[2] and not expired(ticket)
    finally:
        return valid

def expired(ticket):
    delta = datetime.now() - getdate(ticket)
    return delta.total_seconds() > session_timeout

def getusername(ticket):
    header, payload, valid = jwt.jwt_decode(ticket, jwt_secret, 'HS256')
    payload = json.loads(payload)
    username = payload['user']
    return username

def deleteexpiredsessionfiles():
    for sessionfile in glob.glob(tempfile.gettempdir() + "/" + fileprefix + '*'):
        filedate = datetime.fromtimestamp(os.path.getmtime(sessionfile))
        delta = datetime.now() - filedate
        if delta.total_seconds() > session_timeout:
            os.unlink(sessionfile)
