#!/usr/bin/python3

import hashlib
import json
import base64

# JWT API, without import jwt

def jwt_encode(payload, secret, algorithm):
    m = {
        'HS256': hashlib.sha256(secret.encode('utf-8')),
        'HS512': hashlib.sha512(secret.encode('utf-8'))
    }.get(algorithm, hashlib.md5(secret.encode('utf-8')))
    m.update(json.dumps(payload).encode('utf-8'))
    header = '{"alg":"' + algorithm + '","typ":"JWT"}'
    header = base64.b64encode(header.encode('utf-8')).decode('utf-8')
    payload = base64.b64encode(json.dumps(payload).encode('utf-8')).decode('utf-8')
    token = base64.b64encode(m.digest()).decode('utf-8')
    return header + '.' + payload + '.' + token

def jwt_decode(token, secret, algorithm):
    m = {
        'HS256': hashlib.sha256(secret.encode('utf-8')),
        'HS512': hashlib.sha512(secret.encode('utf-8'))
    }.get(algorithm, hashlib.md5(secret.encode('utf-8')))
    header,payload,hash = token.split('.')
    header = base64.b64decode(header).decode('utf-8')
    payload = base64.b64decode(payload).decode('utf-8')
    hash = base64.b64decode(hash)
    m.update(payload.encode('utf-8'))
    return header, payload, m.digest() == hash

# Usage

# secret = 'nasty-secret'
# payload = {'user': 'jm', 'role': 1}
# token = jwt_encode(payload, secret, 'HS256')


# print(token)
# header, payload, success = jwt_decode(token, secret, 'HS256')
# print(header)
# print(payload)
# print(success)
