#!/usr/bin/python3

import json
import os
import psycopg2
import sys
from psycopg2.extras import RealDictCursor

def translateColToPg(column_name): # translate store-names to postgres-columnnames (currently no translation required for roles)
    return column_name

def translateValToPg(column_name, value):
    if (not value is None) and (str(value).strip() != ''):
        if column_name == 'roleid':
            return str(value).strip()
        else:
            return "'" + str(value).strip() + "'"
    else:
        return "NULL"

def updateRoles(roleInserts, roleUpdates):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        sql = "BEGIN;"
        query.execute(sql)
        roles = {}
        for roleid in roleInserts.keys():
            sql = ""
            inserts = roleInserts[roleid]
            for column_name in inserts.keys():
                if sql == "":
                    sql += "INSERT INTO app.role ("
                else:
                    sql += ", "
                sql += translateColToPg(column_name)
            sql += ") VALUES ("
            first = True
            for column_name in inserts.keys():
                if first:
                    first = False
                else:
                    sql += ", "
                sql += translateValToPg(column_name, inserts[column_name])
            sql += ") RETURNING roleid;"
            query.execute(sql)
            roleid = query.fetchone()['roleid']
            role = {}
            role['roleid'] = roleid
            roles[roleid] = role
        for roleid in roleUpdates.keys():
            sql = ""
            updates = roleUpdates[roleid]
            for column_name in updates.keys():
                if sql == "":
                    sql += "UPDATE app.role SET "
                else:
                    sql += ", "
                sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, updates[column_name])
            sql += " WHERE roleid=" + str(roleid) + ";"
            query.execute(sql)
            role = {}
            role['roleid'] = roleid # optional
            roles[roleid] = role
        messages = list(roles.values())
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys roles!")
    return success, messages

def updateRole(roleid, updates):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        sql = "BEGIN;"
        query.execute(sql)
        for update in updates:
            itemid = update['itemid']
            if 'selected' in update:
                selected = update['selected']
                sql = ""
                if selected:
                    if 'readwrite' in update:
                        readwrite = "TRUE" if update['readwrite'] else "FALSE"
                    else:
                        readwrite = "FALSE"
                    sql = "INSERT INTO app.role_navtree(roleid,itemid,readwrite) SELECT " + str(roleid) + "," + str(itemid) + "," + readwrite + " WHERE NOT EXISTS (SELECT roleid,itemid FROM app.role_navtree WHERE roleid=" + str(roleid) + " AND itemid=" + str(itemid) + ")"
                    query.execute(sql)
                    # if this is the first leaf of a group, also add the parent(s)
                    sql = "SELECT parentid FROM app.navtree WHERE itemid=" + str(itemid)
                    query.execute(sql)
                    itemid = query.fetchone()['parentid']
                    while itemid > 0:
                        sql = "INSERT INTO app.role_navtree(roleid,itemid) SELECT " + str(roleid) + "," + str(itemid) + " WHERE NOT EXISTS (SELECT roleid,itemid FROM app.role_navtree WHERE roleid=" + str(roleid) + " AND itemid=" + str(itemid) + ")"
                        query.execute(sql)
                        sql = "SELECT parentid FROM app.navtree WHERE itemid=" + str(itemid)
                        query.execute(sql)
                        itemid = query.fetchone()['parentid']
                else:
                    sql = "DELETE FROM app.role_navtree WHERE roleid=" + str(roleid) + " AND itemid=" + str(itemid)
                    query.execute(sql)
                    # if this was the last leaf of a group, delete the empty parent(s)
                    sql = "SELECT parentid FROM app.navtree WHERE itemid=" + str(itemid)
                    query.execute(sql)
                    itemid = query.fetchone()['parentid']
                    while itemid > 0:
                        sql = "SELECT count(*) FROM app.role_navtree WHERE roleid=" + str(roleid) + " AND itemid IN (SELECT itemid FROM app.navtree WHERE parentid=" + str(itemid) + ")"
                        query.execute(sql)
                        children = query.fetchone()['count']
                        if children > 0:
                            break
                        sql = "DELETE FROM app.role_navtree WHERE roleid=" + str(roleid) + " AND itemid=" + str(itemid)
                        query.execute(sql)
                        sql = "SELECT parentid FROM app.navtree WHERE itemid=" + str(itemid)
                        query.execute(sql)
                        itemid = query.fetchone()['parentid']
            elif 'readwrite' in update:
                readwrite = "TRUE" if update['readwrite'] else "FALSE"
                sql = "UPDATE app.role_navtree SET readwrite=" + readwrite + " WHERE roleid=" + str(roleid) + " AND itemid=" + str(itemid)
                query.execute(sql)
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys roles!")
    return success, messages

contentlength = os.environ.get('CONTENT_LENGTH', 0)
postdata = sys.stdin.read(int(contentlength))
postdata = json.loads(postdata)
querystring = os.environ.get('QUERY_STRING')
querystring = querystring.split('&')
query = {}
for qs in querystring:
    qs = qs.split('=')
    if len(qs) < 2:
        continue
    query[qs[0]] = qs[1]
if 'roleid' in query:
    roleid = query['roleid']
    success, messages = updateRole(roleid, postdata)
else:
    roleUpdates = {}
    roleInserts = {}
    for record in postdata:
        if 'roleid' in record: # update or insert
            roleid = record['roleid']
            if roleid != -1: # update
                updates = {}
                for key in record.keys():
                    if key != 'roleid':
                        updates[key] = record[key]
                roleUpdates[roleid] = updates
            else: # insert; only one insert is allowed at a time
                inserts = {}
                for key in record.keys():
                    if key != 'roleid':
                        inserts[key] = record[key]
                roleInserts[roleid] = inserts
    success, messages = updateRoles(roleInserts, roleUpdates)

print ("Content-type: application/json")
print ()
if success:
    print("{\"success\": true, \"messages\": " + json.dumps(messages) + "}")
else:
    print("{\"success\": false, \"messages\": " + json.dumps(messages) + "}")

