#!/usr/bin/python3

import json
import os
import psycopg2
import sys
from psycopg2.extras import RealDictCursor
from decode import token_decode

def translateColToPg(column_name): # translate store-names to postgres-columnnames (currently no translation required for users)
    if column_name == 'passwdreset':
        column_name = 'passwd'
    return column_name

def translateValToPg(column_name, value):
    if (not value is None) and (str(value).strip() != ''):
        if column_name == 'userid' or column_name == 'roleid':
            return str(value).strip()
        elif column_name == 'passwd':
            return "'" + token_decode(value) + "'"
        elif column_name == 'passwdreset':
            return "''"
        else:
            return "'" + str(value).strip() + "'"
    else:
        return "NULL"

def updateUsers(userInserts, userUpdates):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        sql = "BEGIN;"
        query.execute(sql)
        users = {}
        for userid in userInserts.keys():
            sql = ""
            inserts = userInserts[userid]
            if not 'passwd' in inserts.keys(): # new user ==> blank passwd
                inserts['passwdreset'] = True
            for column_name in inserts.keys():
                if sql == "":
                    sql += "INSERT INTO app.user ("
                else:
                    sql += ", "
                sql += translateColToPg(column_name)
            sql += ") VALUES ("
            first = True
            for column_name in inserts.keys():
                if first:
                    first = False
                else:
                    sql += ", "
                sql += translateValToPg(column_name, inserts[column_name])
            sql += ") RETURNING userid;"
            query.execute(sql)
            userid = query.fetchone()['userid']
            user = {}
            user['userid'] = userid
            users[userid] = user
            sql = "INSERT INTO app.user_spatialfilter (userid, gid) VALUES (" + str(userid) + ",1);"
            query.execute(sql) # new user ==> set spatialfilter to "world"
        for userid in userUpdates.keys():
            sql = ""
            updates = userUpdates[userid]
            for column_name in updates.keys():
                if sql == "":
                    sql += "UPDATE app.user SET "
                else:
                    sql += ", "
                sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, updates[column_name])
            sql += " WHERE userid=" + str(userid) + ";"
            query.execute(sql)
            user = {}
            user['userid'] = userid # optional
            users[userid] = user
        messages = list(users.values())
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys users!")
    return success, messages

contentlength = os.environ.get('CONTENT_LENGTH', 0)
postdata = sys.stdin.read(int(contentlength))
postdata = json.loads(postdata)
userUpdates = {}
userInserts = {}
for record in postdata:
    if 'userid' in record: # update or insert
        userid = record['userid']
        if userid != -1: # update
            updates = {}
            for key in record.keys():
                if key != 'userid':
                    updates[key] = record[key]
            userUpdates[userid] = updates
        else: # insert; only one insert is allowed at a time
            inserts = {}
            for key in record.keys():
                if key != 'userid':
                    inserts[key] = record[key]
            userInserts[userid] = inserts

success, messages = updateUsers(userInserts, userUpdates)

print ("Content-type: application/json")
print ()
if success:
    print("{\"success\": true, \"messages\": " + json.dumps(messages) + "}")
else:
    print("{\"success\": false, \"messages\": " + json.dumps(messages) + "}")

