#!/usr/bin/python3

import json
import os
import cgi
import psycopg2
import sys
from psycopg2.extras import RealDictCursor
import urllib

def translateColToPg(column_name): # js to postgres
    if column_name == 'startdate':
        column_name = 'start'
    return column_name

def translateValToPg(column_name, value):
    if (not value is None) and (str(value).strip() != ''):
        if column_name == 'low' or column_name == 'a_coef' or column_name == 'b_coef' or column_name == 'c_coef':
            return str(value).strip()
        else:
            return "'" + str(value).strip() + "'"
    else:
        return "NULL"

def updateRatingCurves(locationid, periods, segments):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        periodkey_remap = {}
        sql = "BEGIN;"
        query.execute(sql)
        # period inserts/updates
        for period in periods:
            periodkey = period.get('periodkey')
            sql = ""
            if periodkey >= 100000: #insert
                sql = "SELECT locationkey FROM fews.locations WHERE id='" + locationid + "'";
                query.execute(sql)
                locationkey = query.fetchone()['locationkey']
                sql = "";
                for column_name in period.keys():
                    if column_name == 'periodkey' or column_name == 'is_valid':
                        continue
                    if sql == "":
                        sql += "INSERT INTO ratingcurve.period (locationkey, "
                    else:
                        sql += ", "
                    sql += translateColToPg(column_name)
                sql += ") VALUES (" + str(locationkey)
                for column_name in period.keys():
                    if column_name == 'periodkey' or column_name == 'is_valid':
                        continue
                    sql += ", " + translateValToPg(column_name, period[column_name])
                sql += ") RETURNING periodkey;"
                query.execute(sql)
                new_periodkey = query.fetchone()['periodkey']
                periodkey_remap[periodkey] = new_periodkey
            else: # update
                for column_name in period.keys():
                    if column_name == 'periodkey' or column_name == 'is_valid':
                        continue
                    if sql == "":
                        sql += "UPDATE ratingcurve.period SET "
                    else:
                        sql += ", "
                    sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, period[column_name])
                sql += " WHERE periodkey=" + str(periodkey) + ";"
                query.execute(sql)
            
        # segment inserts/updates
        for segment in segments:
            periodkey = segment.get('periodkey')
            if periodkey in periodkey_remap:
                periodkey = periodkey_remap[periodkey]
            segmentkey = segment.get('segmentkey')
            sql = ""
            if segmentkey >= 100000: #insert
                for column_name in segment.keys():
                    if column_name == 'segmentkey' or column_name == 'periodkey' or column_name == 'is_valid':
                        continue
                    if sql == "":
                        sql += "INSERT INTO ratingcurve.segment (periodkey, "
                    else:
                        sql += ", "
                    sql += translateColToPg(column_name)
                sql += ") VALUES (" + str(periodkey)
                for column_name in segment.keys():
                    if column_name == 'segmentkey' or column_name == 'periodkey' or column_name == 'is_valid':
                        continue
                    sql += ", " + translateValToPg(column_name, segment[column_name])
                sql += ") RETURNING segmentkey;"
                query.execute(sql)
            else: # update
                for column_name in segment.keys():
                    if column_name == 'segmentkey' or column_name == 'periodkey' or column_name == 'is_valid':
                        continue
                    if sql == "":
                        sql += "UPDATE ratingcurve.segment SET "
                    else:
                        sql += ", "
                    sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, segment[column_name])
                sql += " WHERE segmentkey=" + str(segmentkey) + " and periodkey=" + str(periodkey) + ";"
                query.execute(sql)
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys rating curves for station '" + locationid + "'!")
    return success, messages

# main()

# GET/POST
querystring = os.environ.get('QUERY_STRING')
querystring = querystring.split('&')
query = {}
for qs in querystring:
    qs = qs.split('=')
    if len(qs) > 1:
        query[qs[0].lower()] = qs[1]
periods = query.get('periods')
if periods is None: # POST
    contentlength = os.environ.get('CONTENT_LENGTH', 0)
    contentlength = int(contentlength)
    if contentlength > 0:
        querystring = sys.stdin.read(contentlength)
        querystring = querystring.split('&')
        query = {}
        for qs in querystring:
            qs = qs.split('=')
            if len(qs) > 1:
                query[qs[0].lower()] = qs[1]
        periods = query.get('periods')
segments = query.get('segments')
locationid = query.get('locationid')
periods = urllib.parse.unquote(periods)
periods = json.loads(periods)
segments = urllib.parse.unquote(segments)
segments = json.loads(segments)
locationid = urllib.parse.unquote(locationid)
success, messages = updateRatingCurves(locationid, periods, segments)

print("Content-type: application/json")
print()
if success:
    print("{\"success\": true, \"messages\": " + json.dumps(messages) + "}")
else:
    print("{\"success\": false, \"messages\": " + json.dumps(messages) + "}")
